/*--------------------------------------------------------------------------*/
/*																			*/
/*		MCPlayMovie.														*/
/*			by John Wang													*/
/*																			*/
/*		Description:	This program demonstrates playing movies with		*/
/*			QuickTime movie controllers.  It also allows the user to save	*/
/*			the movies in a flattened format using FlattenMovie().			*/
/*																			*/
/*		Version:		1.0 Completed 11/17/91								*/
/*						1.1 Added palette support so that app restores		*/
/*							color table when changed by another app.		*/
/*																			*/
/*--------------------------------------------------------------------------*/

#include 	<GestaltEqu.h>
#include	<Palette.h>
#include	"Movies.h"

#define Gestalttest		0xA1AD
#define NoTrap			0xA89F

#define	appleID			128			
#define	appleMenu		0
#define	aboutMeCommand	1

#define	fileID			129
#define openCommand		1
#define	flattenCommand	2
#define closeCommand	3
#define	quitCommand 	5

#define	aboutMeDLOG		128
#define	okButton		1

#define	MAXWINDOWS		5

/*------------------------------------------------------*/
/*	Global Variables.									*/
/*------------------------------------------------------*/

Boolean				DoneFlag = FALSE;
MenuHandle			mymenu0, mymenu1;
Boolean				playingMovie[MAXWINDOWS];
Movie				myMovie[MAXWINDOWS];
WindowPtr			myWindow[MAXWINDOWS];
MovieController		mcPlay[MAXWINDOWS];
int					startlocation;
CTabHandle			mycolors;
PaletteHandle		srcPalette;

/*------------------------------------------------------*/
/*	getMovieFromFile().									*/
/*------------------------------------------------------*/

Movie getMovieFromFile()
{
	OSErr					err;
	StandardFileReply		reply;
	Point					where = {200, 50};
	SFTypeList				types;
	short					movieResRefNum;
	short					actualResId;
	Movie					theMovie;
	
	types[0] = 'MooV';
	StandardGetFilePreview(nil, 1, types, &reply);
	if (!reply.sfGood) return((Movie) 0);

	err = OpenMovieFile(&reply.sfFile, &movieResRefNum, fsRdPerm);
	if (GetMoviesError()) return((Movie) 0);
	if (err) return ((Movie) 0);
	actualResId = DoTheRightThing;

	err = NewMovieFromFile(&theMovie, movieResRefNum, &actualResId, (char *) 0, newMovieActive, (Boolean *) 0);
	if (GetMoviesError()) return((Movie) 0);
	if (err) return ((Movie) 0);

	err = CloseMovieFile(movieResRefNum);
	if (GetMoviesError()) return((Movie) 0);
	if (err) return ((Movie) 0);

	return (theMovie);
}

/*------------------------------------------------------*/
/*	playMovie().											*/
/*------------------------------------------------------*/

OSErr playMovie(int	index)
{
	Rect			movieBounds;
	
	GetMovieBox(myMovie[index], &movieBounds);
	OffsetRect(&movieBounds, -movieBounds.left, -movieBounds.top);
	SetMovieBox(myMovie[index], &movieBounds);
	OffsetRect(&movieBounds, startlocation, startlocation);

	

	myWindow[index] = NewCWindow(0L, &movieBounds, "\PMovie!", 1, 0, (WindowPtr) -1, FALSE, 0L);
	SetPalette ((WindowPtr) myWindow[index], srcPalette, TRUE);

	OffsetRect(&movieBounds, -startlocation, -startlocation);
	startlocation += 50;
	if (startlocation > 300) startlocation = 50;
	
	SetMovieGWorld(myMovie[index], (CGrafPtr) myWindow[index], 0);
	if (GetMoviesError()) DebugStr("\PSetMovieGWorld error.");;
	mcPlay[index] = NewMovieController(myMovie[index], &movieBounds, mcTopLeftMovie);
	MCGetControllerBoundsRect(mcPlay[index], &movieBounds);
	SizeWindow(myWindow[index], movieBounds.right - movieBounds.left,
							movieBounds.bottom - movieBounds.top, TRUE);
/*	Uncomment these lines if you want to pre load the movie into ram.
	Don't forget to increase the partition size when recompiling.
	
	GotoBeginningOfMovie(myMovie[index]);
	if (LoadMovieIntoRam(myMovie[index], GetMovieTime(myMovie[index], 0L),
									GetMovieDuration(myMovie[index]),
									0) != noErr)
		DebugStr("\PNot enough memory to load movie into ram.");
*/
}

/*------------------------------------------------------*/
/*	flatten().											*/
/*------------------------------------------------------*/

short flatten(int index)
{
	StandardFileReply		reply;
	OSErr       theErr      = noErr;

	StandardPutFile("\PName of flattened movie.", "\PUntitled", &reply);
	if (!reply.sfGood) return;

 	if (theErr = GetMoviesError()) 
		DebugStr("\PCall Before FlattenMovies failed.");
 
	FlattenMovie(myMovie[index],
				flattenAddMovieToDataFork,
				&reply.sfFile,
				'JWJW',
				0,
				createMovieFileDeleteCurFile,
				nil,
				nil);
				
	if (theErr = GetMoviesError()) 
		DebugStr("\PFlattenMovies failed.");
  
	return(theErr);
}

/*------------------------------------------------------*/
/*	showAboutMeDialog()									*/
/*------------------------------------------------------*/

void showAboutMeDialog()
{
	GrafPtr 	savePort;
	DialogPtr	theDialog;
	short		itemHit;

	GetPort(&savePort);
	theDialog = GetNewDialog(aboutMeDLOG, nil, (WindowPtr) -1);
	SetPort(theDialog);

	do {
		ModalDialog(nil, &itemHit);
	} while (itemHit != okButton);

	CloseDialog(theDialog);

	SetPort(savePort);
	return;
}

/*------------------------------------------------------*/
/*	init().												*/
/*------------------------------------------------------*/

void init()
{
	OSErr				err;
	int					i;
	long				QDfeature, OSfeature;

	/*	Initialize Managaer.	*/
	InitGraf(&qd.thePort);
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitDialogs(nil);
	InitCursor();

	/*	Set up menus.	*/
	mymenu0 = GetMenu(appleID);
	AddResMenu(mymenu0, 'DRVR');
	InsertMenu(mymenu0,0);
	mymenu1 = GetMenu(fileID);
	InsertMenu(mymenu1,0);
	DrawMenuBar();
	
	/*	Set up variables.	*/
	startlocation = 50;
	for (i = 0; i < MAXWINDOWS; i++) {
		playingMovie[i] = FALSE;
		myWindow[i] = nil;
	}
	mycolors = GetCTable (72);
	if (mycolors == nil) Debugger();
	srcPalette = NewPalette (((**mycolors).ctSize)+1, mycolors, pmTolerant, 0);

	/*	Use Gestalt to find if QuickDraw and QuickTime is available.	*/
	if ((GetTrapAddress(Gestalttest) != GetTrapAddress(NoTrap))) {
		err = Gestalt(gestaltQuickdrawVersion, &QDfeature);
		if (err)
			ExitToShell();
		err = Gestalt(gestaltSystemVersion, &OSfeature);
		if (err)
			ExitToShell();
		if (!DoneFlag && (QDfeature & 0x0f00) != 0x0200 && OSfeature < 0x0607)
			ExitToShell();
		err = Gestalt(gestaltQuickTime, &QDfeature);
		if (err)
			ExitToShell();
	} else
		ExitToShell();
	
	/*	Open QuickTime last.	*/
	err = EnterMovies();
	if (err)
		ExitToShell();
}

/*------------------------------------------------------*/
/*	finish().											*/
/*------------------------------------------------------*/

void finish()
{
	int i;

	for (i = 0; i < MAXWINDOWS; i++)
		if (playingMovie[i] == TRUE) {
			CloseComponent(mcPlay[i]);
			DisposeMovie(myMovie[i]);
			DisposeWindow(myWindow[i]);
 			playingMovie[i] = FALSE;
		}
	
	ExitMovies();
	ExitToShell();
}

/*------------------------------------------------------*/
/*	doOpenCommand().										*/
/*------------------------------------------------------*/

void doOpenCommand()
{
	int			i, useThisIndex;
	
	useThisIndex = -1;
	
	/*	Search for the first window that is nil.	*/
	for (i = MAXWINDOWS-1; i >= 0; i--)
		if (myWindow[i] == nil)
			useThisIndex = i;
			
	/*	If index = -1, then it means that there are no windows avaiable.	*/
	if (useThisIndex != -1) {
		myMovie[useThisIndex] = getMovieFromFile();
		if (myMovie[useThisIndex] != 0) {
			playMovie(useThisIndex);
			playingMovie[useThisIndex] = TRUE;
		}
	}
}

/*------------------------------------------------------*/
/*	doFlattenCommand().										*/
/*------------------------------------------------------*/

void doFlattenCommand()
{
	int			i, useThisIndex;
	WindowPtr	myTempWindow;
	
	/*	Flatten movie that is currently selected.	*/
	myTempWindow = FrontWindow();
	if (myTempWindow == nil) return;
	for (i = 0; i < MAXWINDOWS; i++)
		if (myWindow[i] == myTempWindow)
			flatten(i);
}

/*------------------------------------------------------*/
/*	doCloseCommand().										*/
/*------------------------------------------------------*/

void doCloseCommand()
{
	int		i, useThisIndex;
	WindowPtr	myTempWindow;

	/*	Close selected window.	*/
	myTempWindow = FrontWindow();
	if (myTempWindow == nil) return;
	for (i = 0; i < MAXWINDOWS; i++)
		if (myWindow[i] == myTempWindow) {
			CloseComponent(mcPlay[i]);
			DisposeMovie(myMovie[i]);
			DisposeWindow(myTempWindow);
			playingMovie[i] = FALSE;
			myWindow[i] = nil;
		}
}

/*------------------------------------------------------*/
/*	doCommand().										*/
/*------------------------------------------------------*/

void doCommand(mResult)
	long	mResult;
{
	int 					theMenu, theItem;
	char					daName[256];
	GrafPtr 				savePort;

	theItem = LoWord(mResult);
	theMenu = HiWord(mResult);
	
	switch (theMenu) {
		case appleID:
			if (theItem == aboutMeCommand)
				showAboutMeDialog();
			else {
				GetItem(mymenu0, theItem, daName);
				GetPort(&savePort);
				(void) OpenDeskAcc(daName);
				SetPort(savePort);
			}
			break;

		case fileID:
			switch (theItem) {
				case openCommand:
					doOpenCommand();
					break;
				case flattenCommand:
					doFlattenCommand();
					break;
				case closeCommand:
					doCloseCommand();
					break;
				case quitCommand:
					DoneFlag = TRUE;
					break;
				default:
					break;
				}
			break;
	}
	HiliteMenu(0);
	return;
}

/*------------------------------------------------------*/
/*	playMovies().											*/
/*------------------------------------------------------*/

int playMovies(EventRecord *myEvent)
{
	int		i;
	
	for (i = 0; i < MAXWINDOWS; i++)
		if (playingMovie[i] == TRUE) {
			if (MCIsPlayerEvent(mcPlay[i], myEvent))
				return(TRUE);
		}
	return(FALSE);
}

/*------------------------------------------------------*/
/*	main().												*/
/*------------------------------------------------------*/

main()
{
	int					i;
	char			key;
	Boolean			track;
	long			growResult;
	EventRecord 	myEvent;
	WindowPtr		whichWindow;
	int				yieldTime;
	int				ret;

	init();
	yieldTime = 0;
	for ( ;; ) {
	
		/*	We can't just do ExitToShell because we must cann ExitMovies.	*/
		if (DoneFlag)
			finish();
			
		ret = WaitNextEvent(everyEvent, &myEvent, yieldTime, nil);
		
		if (playMovies(&myEvent))
				continue;
				
		if (ret) {
		
			switch (myEvent.what) {
				case mouseDown:
					switch (FindWindow(myEvent.where, &whichWindow)) {
						case inSysWindow:
							SystemClick(&myEvent, whichWindow);
							break;
						case inMenuBar:
							doCommand(MenuSelect(myEvent.where));
							break;
						case inContent:
							SelectWindow(whichWindow);
							break;
						case inDrag:
							DragWindow (whichWindow, myEvent.where, &qd.screenBits.bounds);
							break;
						case inGrow:
							break;
						case inGoAway:
							track = TrackGoAway (whichWindow, myEvent.where);
							if (track)	doCloseCommand();
							break;
						case inZoomIn:
							break;
						case inZoomOut:
							break;
						default:
							break;
					}
					break;
				case keyDown:
				case autoKey:
					key = myEvent.message & charCodeMask;
					if ( myEvent.modifiers & cmdKey )
						if ( myEvent.what == keyDown )
							doCommand(MenuKey(key));
					break;
				case updateEvt:
					for (i = 0; i < MAXWINDOWS; i++)
						if ((WindowPtr) myEvent.message == myWindow[i]) {
							BeginUpdate((WindowPtr) myWindow[i]);
							EndUpdate((WindowPtr) myWindow[i]);
						}
					break;
				case diskEvt:
					break;
				case activateEvt:
					break;
				case app4Evt:
					break; 
				default:
					break;
			}
		}
	}
}